"""
pytest commands:
  pytest tests -- test all the tests directory test files
  pytest -s tests -- print is work
  pytest tests/test_help.py -- test specific file
  pytest tests/test_help.py::test_sign_get -- test specific function
"""

test_cookie = "a1=1886132f6cav7mfdpcid2mnbahv45uuvea5z3lvji30000329042;webId=b360cce06e1bcad5e48dafb7b612265e;gid=yYYKyqJiYydqyYYKyqJiK01hS0hW6ifUS3fJ2V36ID0xhuq84277hq888qJj84J8ji8dW2S0;gid.sign=3gVA5Ji2C13wAa8svTrYyqcJDc8=;webBuild=2.11.5;acw_tc=0ab967f98018c20dd6b0745ee8a1596c3750bf84d2f8727a93f5c10315054162; web_session=040069b253793fdd9ccd5842a2364b86586515;galaxy_creator_session_id=S5mv0vWIWqrCX2GSJK9LFaudFrUkNq7TYW3b;galaxy.creator.beaker.session.id=1687620188771003075230;xsecappid=creator-creator;websectiga=2a3d3ea002e7d92b5c9743590ebd24010cf3710ff3af8029153751e41a6af4a3;sec_poison_id=f93e48b6-3162-4f6f-9984-5fcd908695cb"
